% Dette er hovedfila Tofase for systemet uten tilbakekobling.
% Denne rutinen er utviklet i prosjektet, og baserer seg p 
% (Jahanshahi, 2013). 
% Her vises hovedvinduet for systemet uten tilbakekobling
% Her hentes tre andre rutiner: 'Miniloop_parameters()', 
% 'MiniLoop_initialize(u0,par)' og 'Loop_model'.

clear all;
close all;
clc;


%%Definerer globale variable for  kunne hente ut figurer med 
% tid p x-aksen. 

global z1_plot QQ_out_plot Alpha_Lmt_plot Alpha_Lt_plot 
global A_g_plot A_l_plot 


% Henter verdier fra filen MiniLoop_parameters() 

par =  MiniLoop_parameters();


% Her settes simuleringstid Tf, initell ventilpning z0, samt 
% at innstrmningsratene QG_in og QL_in settes konstante. 
% Innstrmningsratene regnes om fra [l/min] til [kg/s]. 
% Deretter settes verdiene inn i en 3x1 pdragsmatrise u0 . 
% Tu lager et array fra 0 til Tf med like stor steglengde.

Tf=500; 
tu_size = Tf+1;
tu = linspace(0,Tf,tu_size); 
z0=12;
QG_in = 4.5; %[l/min]
QL_in = 4;   %[l/min]
wG_in0 = par.Rog_n*QG_in/(60*1000); %kg/s
wL_in0 = par.rho_L*QL_in/(60*1000); %kg/s
u0 = [z0;wG_in0;wL_in0];


% ******** Finner initalverdier *************
% Her kjres initaliseringsfilen MiniLoop_Initialize. 
% Initialiseringsfilen bruker parametere fra parameterfilen 
% MiniLoop_parameters og fra matrisen u0. Nr systemet er ferdig 
% initalisert lages det en pdragsmatrise med lik lengde som antall 
% steg systemet skal kjres. slik at u blir 3x(size(tu)) matrise. 
% Hvor tu er antall steg systemet kjres. 
 
disp('Initialiserer...')
tic  
[x0,y0,par] =  MiniLoop_initialize(u0,par); 
toc
disp('Ferdig initialisert...')


% Kjrer flere mindre sprang i ventilpning z1, som er plassert 
% i u0(1). Her kjres et sprang etter 100 sekunder. 

u =u0*ones(size(tu)); 


 for i = 1:tu_size
     if i < 100*Tf/tu_size 
         u(1,i) = u0(1);        
     else
         u(1,i) = u0(1)+1;
     end
 end
 
 
% Her beregnes tilstandens verdier ved  bruke en ode23s-funksjon. 
% Denne metoden passer et system som er 'stiff' og baserer seg p 
% en modifisert Rosenbrock formel av andre orden. 
% ode23s returnerer tiden T og verdiene p tilstandene X

options = odeset ('InitialStep', 1.0e-1, 'MaxStep', 1.0e-1,'RelTol', 1.0e-6, 'AbsTol', 1.0e-6);
[T,X] = ode23s(@(t,x)Loop_model(t,x,tu,u,par),[0 Tf],x0,options);


% Beregner trykk i pipeline (P1) og toppen av riser (P2_t) 
% til plotting av figurer.

P1 = X(:,1)*par.R*par.T1./(par.M_G*(par.V1 + par.Vb - X(:,2)/par.rho_L));

V_G2 = par.V2 - X(:,4)/par.rho_L;
P2_t = X(:,3)*par.R*par.T2./(par.M_G*V_G2);
P2_t = max(P2_t,par.P0);


% Plotter figur 1 og figur 2 
figure(1)
plot(T,P1)

figure(2)
grid on
[haxes,hline1,hline2] = plotyy(T,P1,T,P2_t,'plot','plot');
ylabel(haxes(1),'P1 [Pa]') % Navnsetting av venstre y-akse i figuren
ylabel(haxes(2),'P2_t [Pa]') % Navnsetting av hyre y-akse i figuren
xlabel(haxes(2),'Tid [sek]') % Navnsetting av x-akse i figuren
legend('y = P1','y = P2_t')





     