
% Dette er modellrutinen Loop_model for systemet med trykkregulering.
% Denne rutinen er utviklet i prosjektet, og baserer seg p 
% (Jahanshahi, 2013). Her vises modellvinduet for det 
% trykkregulerte systemet. Her beregnes systemets 
% dynamikk gjennom hele simuleringen.


function dxdt = Loop_model(t,x,tu,u,par,P_ref,K_p) 


%Definerer globale variable for  kunne hente ut figurer med 
%tid p x-aksen. 

global z1_plot 


% Her vises systemets 4 tilstander x1 - x4, og i tillegg oppdateres 
% integralet int og ventilpningen z1 ved hver gjennomkjring av 
% denne rutinen. 
  
x1 = x(1);  %(m_G1)  Gassmasse i rrledning
x2 = x(2);  %(m_L1)  Vskemasse i rrledning
x3 = x(3);  %(m_G2)  Gassmasse i stigerr
x4 = x(4);  %(m_L2)  Vskemasse i stigerr
int = x(5); 
z1 = x(6);


% Ved  bruke interpl(a,b,c), fr alle matrisene u1, u2, u3, P_ref1 og 
% K_p1 lik lengde i a og b som i c. 
 
u1 = interp1(tu,u(1,:),t);  
u2 = interp1(tu,u(2,:),t);  
u3 = interp1(tu,u(3,:),t);
P_ref1 = interp1(tu,P_ref(1,:),t);
K_p1 = interp1(tu,K_p(1,:),t);


% Beregner trykket i rrledningen (P1) og p toppen av stigerret (P2_t)

P1 = x1*par.R*par.T1./(par.M_G*(par.V1 + par.Vb - x2/par.rho_L));
V_G2 = par.V2 - x4/par.rho_L;
P2_t = x3*par.R*par.T2./(par.M_G*V_G2); 
P2_t = max(P2_t,par.P0);


% To pdrag: Masseinnstrmning av gass w_G_in og masseinstrmning
% av vske w_L_in:

w_G_in = u2; 
w_L_in = u3;


% Beregner vskenivet i rrledningen h1 med bruk av steady-
% state vskeniv i rrledning h1ss, stasjonr vskemasse i 
% rrledningen x2ss, og konstanten K. 

h1ss = par.k_h*par.Alpha_L1_av*par.hc;
x2ss = par.V1*par.rho_L*par.Alpha_L1_av; 
K = sin(par.theta)/(par.A1*(1-par.Alpha_L1_av)*par.rho_L);
h1 = max(h1ss + K*(x2 - x2ss),0); 

V_G2 = par.V2 - x4/par.rho_L;

P1 = x1*par.R*par.T1/(par.M_G*(par.V1 + par.Vb - x2/par.rho_L));
rho_G1 = x1/(par.V1 + par.Vb - x2/par.rho_L);


% Parametere i rrledning. Beregner friksjon i rrledning Fric_pipe 
% og areal av gass A_g og vske A_l

Uslin = w_L_in/(par.A1*par.rho_L);  
Re1 = par.rho_L*Uslin*(2*par.r1)/par.visl; 

temp = -1.8*log10((par.eps/par.D1/3.7)^1.11+6.9/Re1);
Lambda1 = (1/temp)^2; 

Fric_pipe = par.Alpha_L1_av*Lambda1*par.rho_L*Uslin^2*par.L1/(2*par.r1); 

A_g = (h1<par.hc)*(par.A1/par.hc^2)*(par.hc - h1)^2;
A_l = par.A1 - A_g;


% Parametere i riser. Beregner friksjon i riser (Fric_riser), og 
% trykket i bunn av riser (P2_b)

rho_G2 = x3/V_G2;
Alpha_L2_av = x4/(par.rho_L*par.V2);
rho_mix_av = (x3+x4)/par.V2;

Usl2 = w_L_in/(par.rho_L*par.A2); 
Usg2 = w_G_in/(rho_G2*par.A2); 
Um = Usl2+Usg2;

Re2=rho_mix_av*Um*(2*par.r2)/par.visl;

temp = -1.8*log10((par.eps/par.D2/3.7)^1.11+6.9/Re2);
Lambda2=(1/temp)^2; 

Fric_riser= Alpha_L2_av*Lambda2*rho_mix_av*Um^2*(par.L2+par.L3)/(2*par.r2);

P2_b = P2_t + rho_mix_av*par.g*par.L2 + Fric_riser;


% Her beregnes massestrmning av gass w_G1 og vske w_L1 i rrledningen: 

w_G1 = par.K_g*A_g*sqrt(rho_G1*max(0,P1-Fric_pipe-P2_b));
w_L1 = (P1-Fric_pipe+par.rho_L*par.g*h1-P2_b>0)*...
    par.K_o*A_l*sqrt(par.rho_L*abs(P1-Fric_pipe+par.rho_L*par.g*h1-P2_b));


% Beregner vskeandel ved bunn av stigerr Alpha_Lb, og vskeandel ved 
% toppen av stigerret Alpha_Lt. Dersom Alpha_Lb er mindre enn 
% gjennomsnittlig vskeandel i stigerret Alpha_L2_av, settes 
% Alpha_Lb = Alpha_L2_av. 

Alpha_Lb = 1 -  A_g/par.A1;

if(Alpha_Lb<=Alpha_L2_av) 
    Alpha_Lb=Alpha_L2_av;
end

Alpha_Lt =  2*Alpha_L2_av - Alpha_Lb;

if(Alpha_Lt>Alpha_L2_av)
    Alpha_Lt = Alpha_L2_av;
elseif(Alpha_Lt<0)
    Alpha_Lt = 0;
end


% Her beregnes masseutstrmningen av olje og vske henholdsvis w_G_out 
% og w_L_out. Her er Alpha_Lmt er vskens masseandel, rho_t flerfasens 
% densitet [kg/(m^3)], w_mix_out er flerfasens masseutstrmning [kg/s] 
% ved toppen av stigerret. Q_out er volumutstrmning [m^3/s], mens 
% QQ_out er volumutstrmning i [m^3/t]. w_L_out er utstrmning av vske 
% p utgangen [kg/s], og w_G_out er utstrmning av gass utgangen [kg/s].

Alpha_Lmt = Alpha_Lt*par.rho_L/(Alpha_Lt*par.rho_L + (1-Alpha_Lt)*rho_G2); 
rho_t = Alpha_Lt*par.rho_L + (1-Alpha_Lt)*rho_G2;  

w_mix_out = par.K_pc*z1*sqrt(rho_t*max(0,P2_t-par.P0)); 
Q_out = w_mix_out/rho_t;
QQ_out = Q_out*3600; 

w_L_out = Alpha_Lmt*w_mix_out; %
w_G_out = (1 - Alpha_Lmt)*w_mix_out; %Gassutstrmning p utgang [kg/s]


% Her beregnes pdraget fra PI-trykkregulatoren. K_p1 er 
% proporsjonalforsterkning, T_i1 er integraltiden, T_z  er en
% ventilkonstant, P_ref1 er trykkreferansen og err er avviket mellom
% P_ref og P1. Det totale pdraget u_reg1 er summen av nominelt u1,  
% proporsjonalt u_p, og integralpdraget u_i. 

T_z = 2.5;
err = (P_ref1 - P1);
T_i1 = 0.85*42;

% P-pdrag, u_p: 
u_p = K_p1*err;

% I-pdrag, u_i:

u_i = 1*(K_p1/T_i1)*int;

% Totalt pdrag, u_reg: 
u_reg = u1 + u_p + u_i;  


% Sjekker om pdraget er mellom 0 og 100. Sjekker med andre ord om 
% pdraget svarer til at ventilen skal vre fra helt stengt til helt 
% pen. Deretter settes ventilpdraget z lik en hundredel av pdraget. 
% z er ventilpdraget, ikke ventilpningen.

if u_reg < 0 
    u_reg = 0; % u_reg >= 6.5% blir P1 stabil i 4state
end

if u_reg > 100
    u_reg = 100; 
end

z = u_reg / 100;

% Lager et globalt array slik at det er mulig  plotte en figur som 
% viser ventilpningen. 

 z1_plot(ceil(t+0.01))=z1;

% Dette systemet med trykkregulering sender til slutt ut endringen i de 
% 4 tilstandene, og i tillegg endringen i integralet dint og endringen 
% i ventilpningen dz1. Disse gir inngangsvariable til ode23-funksjonen 
% i Tofase.m 
 
dx1 = w_G_in - w_G1;
dx2 = w_L_in - w_L1; 
dx3 = w_G1 - w_G_out;
dx4 = w_L1 - w_L_out;
dint = err; 
dz1 = (1/T_z)*(z-z1);

dxdt = [dx1;dx2;dx3;dx4;dint;dz1];

end