% Dette er MiniLoop_model0. Denne rutinen kalles i initialiseringsrutinen
% nr stasjonre tilstandsverdier skal beregnes. Denne rutinen er lik for 
% systemet uten tilbakekobling, systemet med trykkregulering og systemet
% med kaskaderegulering. Denne rutinen er hentet fra (Jahanshahi, 2013) og 
% dens innhold er ikke endret foruten om estetiske endringer.  

function [xdot,y] = MiniLoop_model0(x,u,par) 

x1 = x(1);  %(m_G1)
x2 = x(2); %(m_L1)  
x3 = x(3); %(m_G2)
x4 = x(4); %(m_L2)

z = u(1)/100; 
w_G_in = u(2);
w_L_in = u(3);

% Beregner vskenivet i rrledningen h1 med bruk av steady- 
% state vskeniv i rrledning h1ss, stasjonr vskemasse i 
% rrledningen x2ss, og konstanten K.

h1ss = par.k_h*par.Alpha_L1_av*par.hc; 
x2ss = par.V1*par.rho_L*par.Alpha_L1_av;
K = sin(par.theta)/(par.A1*(1-par.Alpha_L1_av)*par.rho_L);
h1 = max(h1ss + K*(x2 - x2ss),0);

V_G2 = par.V2 - x4/par.rho_L;


P1 = x1*par.R*par.T1/(par.M_G*(par.V1 + par.Vb - x2/par.rho_L));
rho_G1 = x1/(par.V1 + par.Vb - x2/par.rho_L);

Uslin = w_L_in/(par.A1*par.rho_L);
Re1=par.rho_L*Uslin*(2*par.r1)/par.visl;


temp = -1.8*log10((par.eps/par.D1/3.7)^1.11+6.9/Re1);
Lambda1=(1/temp)^2; 

Fric_pipe = par.Alpha_L1_av*Lambda1*par.rho_L*Uslin^2*par.L1/(2*par.r1);


P2_t = x3*par.R*par.T2/(par.M_G*V_G2);
P2_t = max(P2_t,par.P0);
rho_G2 = x3/V_G2;
Alpha_L2_av = x4/(par.rho_L*par.V2);
rho_mix_av = (x3+x4)/par.V2;


Usl2 = w_L_in/(par.rho_L*par.A2);
Usg2 = w_G_in/(rho_G2*par.A2);
Um = Usl2+Usg2;

Re2=rho_mix_av*Um*(2*par.r2)/par.visl; 


temp = -1.8*log10((par.eps/par.D2/3.7)^1.11+6.9/Re2);
Lambda2=(1/temp)^2; 

Fric_riser= Alpha_L2_av*Lambda2*rho_mix_av*Um^2*(par.L2+par.L3)/(2*par.r2);


A_g = (h1<par.hc)*(par.A1/par.hc^2)*(par.hc - h1)^2;

A_l = par.A1 - A_g;

P2_b = P2_t + rho_mix_av*par.g*par.L2 + Fric_riser;

w_G1 = par.K_g*A_g*sqrt(rho_G1*max(0,P1-Fric_pipe-P2_b));
w_L1 = (P1-Fric_pipe+par.rho_L*par.g*h1-P2_b>0)*...
    par.K_o*A_l*sqrt(par.rho_L*abs(P1-Fric_pipe+par.rho_L*par.g*h1-P2_b));

Alpha_Lb = 1 -  A_g/par.A1;

if(Alpha_Lb<=Alpha_L2_av) 
    Alpha_Lb=Alpha_L2_av;
end

Alpha_Lt =  2*Alpha_L2_av - Alpha_Lb;


if(Alpha_Lt>Alpha_L2_av)
    Alpha_Lt = Alpha_L2_av;
elseif(Alpha_Lt<0)
    Alpha_Lt = 0;
end


Alpha_Lmt = Alpha_Lt*par.rho_L/(Alpha_Lt*par.rho_L + (1-Alpha_Lt)*rho_G2);
rho_t = Alpha_Lt*par.rho_L + (1-Alpha_Lt)*rho_G2; 

w_mix_out = par.K_pc*z*sqrt(rho_t*max(0,P2_t-par.P0));
Q_out = w_mix_out/rho_t;

w_L_out = Alpha_Lmt*w_mix_out; %Vskeutstrmning p utgang
w_G_out = (1 - Alpha_Lmt)*w_mix_out; %Gassutstrmning p utgang

% Systemets 4 Tilstander
dx1 = w_G_in - w_G1;
dx2 = w_L_in - w_L1;
dx3 = w_G1 - w_G_out;
dx4 = w_L1 - w_L_out;


xdot = [dx1;dx2;dx3;dx4];

y =  [1e-3*(P1-par.Pnormal) %Y1 kPa mlt       Trykk i rrledning
      1e-3*(P2_t-par.Pnormal) %Y2 kPa mlt     Trykk p toppen av stigerr
      1000*60*Q_out          %Y3 litr/min   x 1000 --> m^3 til liter p utgang
      60*w_mix_out];    %Y4 Kg/min;         x 60   --> kg/s til kg/min av mix p utgang
end