
% Dette er modellrutinen Loop_model for systemet med trykkregulering.
% Denne rutinen er utviklet i prosjektet, og baserer seg p 
% (Jahanshahi, 2013). Her beregnes det kaskaderegulerte systemets dynamikk
% gjennom hele simuleringen. 

function dxdt = Loop_model(t,x,tu,u,par,P_ref,K_p) 

%Definerer globale variable for  kunne hente ut figurer med 
% tid p x-aksen. 

global  z1_plot QQ_out_plot Alpha_Lmt_plot Alpha_Lt_plot 
global A_g_plot A_l_plot 

%% Her vises systemets 4 tilstander x1 - x4, og i tillegg oppdateres 
 % integralet til trykkregulator int1, integralet til 
 % strmningsregulator int2 og ventilpningen z1 ved hver 
 % gjennomkjring av denne rutinen. 
 
x1 = x(1);  %(m_G1) Gassmasse i pipeline
x2 = x(2); %(m_L1)  Vskemasse i pipeline
x3 = x(3); %(m_G2)  Gassmasse i stigerr
x4 = x(4); %(m_L2)  Vskemasse i stigerr
int1 = x(5); 
int2 = x(6);
z1 = x(7);



% Ved  bruke interpl(a,b,c), fr alle matrisene u1, u2, u3, P_ref1 og 
% K_p1 lik lengde i a og b som i c. 

u1 = interp1(tu,u(1,:),t);  
u2 = interp1(tu,u(2,:),t);  
u3 = interp1(tu,u(3,:),t);
P_ref1 = interp1(tu,P_ref(1,:),t);
K_p1 = interp1(tu,K_p(1,:),t);



% Beregner trykk i rrledningen (P1) og p toppen av stigerret (P2_t)

P1 = x1*par.R*par.T1./(par.M_G*(par.V1 + par.Vb - x2/par.rho_L));
V_G2 = par.V2 - x4/par.rho_L;
P2_t = x3*par.R*par.T2./(par.M_G*V_G2);
P2_t = max(P2_t,par.P0);

y1 = 1e-3*(P1-par.Pnormal); %Y1 kPa Gauge
y2 = 1e-3*(P2_t-par.Pnormal); %Y2 kPa Gauge



% Setter de to pdragene: gassinstrmning w_G_in og vskeinstrmning 

w_G_in = u2;            
w_L_in = u3;




% Beregner vskenivet i rrledningen h1 med bruk av steady- 
% state vskeniv i rrledning h1ss, stasjonr vskemasse i 
% rrledningen x2ss, og konstanten K.

h1ss = par.k_h*par.Alpha_L1_av*par.hc;
x2ss = par.V1*par.rho_L*par.Alpha_L1_av;
K = sin(par.theta)/(par.A1*(1-par.Alpha_L1_av)*par.rho_L);
h1 = max(h1ss + K*(x2 - x2ss),0); 

V_G2 = par.V2 - x4/par.rho_L;

P1 = x1*par.R*par.T1/(par.M_G*(par.V1 + par.Vb - x2/par.rho_L));
rho_G1 = x1/(par.V1 + par.Vb - x2/par.rho_L);



% Beregner parametere i rrledningen:
% Beregner friksjon i rrledning Fric_pipe og areal av gass A_g, 
% og areal av vske A_l. Uslin er vskens overflatehastighet i 
% rrledningen Re1 er reynholdstallet i rrledningen. Resten er 
% forholdstall.
 
Uslin = w_L_in/(par.A1*par.rho_L);  
Re1=par.rho_L*Uslin*(2*par.r1)/par.visl; 

temp = -1.8*log10((par.eps/par.D1/3.7)^1.11+6.9/Re1);
Lambda1=(1/temp)^2; 

Fric_pipe = par.Alpha_L1_av*Lambda1*par.rho_L*Uslin^2*par.L1/(2*par.r1); 

A_g = (h1<par.hc)*(par.A1/par.hc^2)*(par.hc - h1)^2;
A_l = par.A1 - A_g;



% Beregner parametere i stigerret: 
% Beregner friksjon i stigerret Fric_riser, og trykket i bunn av 
% riser P2_b. Her er Usl2 vskens overflatehastighet i stigerret, Ug2 
% er gassens overflatehastighet i stigerret, Alpha_L2_av er 
% gjennomsnittlig vskeandel i stigerr og Um er flerfasens 
% overflatehastighet. Re2 er reynholdstallet i stigeret. 

rho_G2 = x3/V_G2;
Alpha_L2_av = x4/(par.rho_L*par.V2);
rho_mix_av = (x3+x4)/par.V2;

Usl2 = w_L_in/(par.rho_L*par.A2); 
Usg2 = w_G_in/(rho_G2*par.A2); 
Um = Usl2+Usg2;

Re2=rho_mix_av*Um*(2*par.r2)/par.visl; 

temp = -1.8*log10((par.eps/par.D2/3.7)^1.11+6.9/Re2);
Lambda2=(1/temp)^2; 
Fric_riser= Alpha_L2_av*Lambda2*rho_mix_av*Um^2*(par.L2+par.L3)/(2*par.r2);

P2_b = P2_t + rho_mix_av*par.g*par.L2 + Fric_riser;



% Beregner massestrmning av gass w_G1 og massestrmning av vske w_L1 i 
% rrledningen:

w_G1 = par.K_g*A_g*sqrt(rho_G1*max(0,P1-Fric_pipe-P2_b));
w_L1 = (P1-Fric_pipe+par.rho_L*par.g*h1-P2_b>0)*...
    par.K_o*A_l*sqrt(par.rho_L*abs(P1-Fric_pipe+par.rho_L*par.g*h1-P2_b));



% Beregner vskeandel ved bunn av stigerr Alpha_Lb, og vskeandel ved 
% toppen av stigerret Alpha_Lt. Dersom Alpha_Lb er mindre enn 
% gjennomsnittlig vskeandel i stigerret Alpha_L2_av, s settes 
% Alpha_Lb = Alpha_L2_av. 

Alpha_Lb = 1 -  A_g/par.A1;

if(Alpha_Lb<=Alpha_L2_av) 
    Alpha_Lb=Alpha_L2_av;
end

Alpha_Lt =  2*Alpha_L2_av - Alpha_Lb;

if(Alpha_Lt>Alpha_L2_av)
    Alpha_Lt = Alpha_L2_av;
elseif(Alpha_Lt<0)
    Alpha_Lt = 0;
end



% Her beregnes masseutstrmningen av olje og vske henholdsvis w_G_out 
% og w_L_out. Her er Alpha_Lmt er vskens masseandel, rho_t flerfasens 
% densitet [kg/(m^3)], w_mix_out er flerfasens masseutstrmning [kg/s] 
% ved toppen av stigerret. Q_out er volumutstrmning [m^3/s], mens 
% QQ_out er volumutstrmning i [m^3/t]. 
 
Alpha_Lmt = Alpha_Lt*par.rho_L/(Alpha_Lt*par.rho_L + (1-Alpha_Lt)*rho_G2); 

rho_t = Alpha_Lt*par.rho_L + (1-Alpha_Lt)*rho_G2;

w_mix_out = par.K_pc*z1*sqrt(rho_t*max(0,P2_t-par.P0)); %[kg/s]
Q_out = w_mix_out/rho_t; %[m^3/s]
QQ_out = Q_out*3600; %[m^3/h]
w_L_out = Alpha_Lmt*w_mix_out; %[kg/s]
w_G_out = (1 - Alpha_Lmt)*w_mix_out; %[kg/s]

QG_in = 4.5; %l/min
QL_lin = 4; %l/min
Qp_sp = (QG_in + QL_lin)*60/1000; %[m^3/h]



% Her beregnes pdraget fra PI-trykkregulatoren. K_p1 er 
% proporsjonalforsterkning, T_i1 er integraltiden,, P_ref1 er 
% trykkreferansen og err er avviket mellom P_ref og P1. Det totale 
% pdraget u_reg1 er summen av nominelt u1, proporsjonalt u_p, og 
% integralpdraget u_i. 

err1 = (P_ref1 - P1);

% Integraltid, T_i:  
Ti_1 = 0.85*42;

% P-pdrag, up_1: 
up_1 = K_p1*err1;

% I-pdrag, ui_1:
ui_1 = 1*(K_p1/Ti_1)*int1;

% Totalt pdrag, u_reg:  
u_reg1 = u1 + up_1 + ui_1;  



% Sjekker om pdraget fra trykkregulatoren er mellom 0 og 100. Deretter 
% skaleres pdraget med en skaleringsfaktor sk_faktor for at pdraget 
% skal svare til strmningsreferansen. Dette forklares i rapporten.


if u_reg1 < 0 
    u_reg1 = 0; % u_reg >= 6.5% blir P1 stabil i 4state
end

if u_reg1 > 100
    u_reg1 = 100; 
end

sk_faktor = (0.47 / 0.1126); 
u_reg1 = (u_reg1/100)*sk_faktor;



% Her beregnes pdraget fra PI-strmningsregulatoren. K_p2 er 
% proporsjonalforsterkning, T_i2 er integraltiden, err2 er avviket 
% mellom u_reg1 og QQ_out. K_v er en ventilkonstant, og Tz er 
% ventilens tidskonstant, a er en tuningparameter, og dP_c er 
% trykkfallet over utlpsventilen. Det totale pdraget fra 
% strmningsregulatoren u_reg2 er summen av nominelt pdrag 10, 
% proporsjonalt pdrag up_2 og pdrag fra integralvirkningen ui_2.

 K_v = (3.14 *(par.r2)*(par.r2))/100; %[m^2/%]
 a = 0.5; %Tuningparameter typisk i omrdet 0.3 - 2.0.
 rho_p = rho_t; %[kg/m^3]
 dP_c = (P2_t - par.Pnormal);%[bar]
 T_v = 3; %[sek]
 Tz= 3;

 err2 = u_reg1 - QQ_out;
 
%P-forsterkning, K_p:
 K_p2 = (a/K_v)*sqrt(rho_p/dP_c) ;
 
%Integraltid, Ti2
 Ti_2 = T_v;  
 
% P-pdrag, u_p: 
 up_2 = K_p2*err2;
 
% I-pdrag, u_i:
 ui_2 = 1*(K_p2/Ti_2)*int2;

% Totalt pdrag, u_reg: 
 u_reg2 = 10 + up_2 + ui_2;  

 
 
% Sjekker om pdraget er mellom 0 og 100. Sjekker med andre ord om 
% pdraget svarer til at ventilen skal vre fra helt stengt til helt 
% pen. Deretter settes ventilpdraget z lik en hundredel av
% regulatorpdraget. z er ventilpdraget, ikke ventilpningen. 
 
 if u_reg2 < 0 
     u_reg2 = 0; % u_reg >= 6.5% blir P1 stabil i 4state
end

if u_reg2 > 100
    u_reg2 = 100; 
end

z = u_reg2/100;


% Her presenteres en oversikt over alle de globale variable, og hvordan 
% disse matrisene er definert. Ved  gjre det p denne mten er det 
% mulig  plotte figurer med tid og ikke antall simuleringer p x-aksen 
% nr de plottes fra Tofase.m. 

 Alpha_Lmt_plot(ceil(t+0.01))=Alpha_Lmt;
 Alpha_Lt_plot(ceil(t+0.01))=Alpha_Lt;
 QQ_out_plot(ceil(t+0.01))=QQ_out;
 z1_plot(ceil(t+0.01))=z1;
 A_g_plot(ceil(t+0.01))=A_g;
 A_l_plot(ceil(t+0.01))=A_l;

% Dette systemet med kaskaderegulering sender til slutt ut endringen i 
% de 4 tilstandene, og i tillegg endringen i integralet til 
% trykkreguleringen dint1 og integralet til kaskadereguleringen dint2, 
% samt endringen i ventilpningen dz1. Disse gir inngangsvariable til 
% ode23-funksjonen i Tofase.m 

dx1 = w_G_in - w_G1;
dx2 = w_L_in - w_L1; 
dx3 = w_G1 - w_G_out;
dx4 = w_L1 - w_L_out;
dint1 = err1; 
dint2 = err2;
dz1 = (1/Tz)*(z-z1);

dxdt = [dx1;dx2;dx3;dx4;dint1;dint2;dz1];

end