% Dette er initialiseringsrutinen MiniLoop_initialize. Her beregnes 
% modellens initielle verdier, basert p verdiene i pdragsmatrisen
% u, og parameterne. Pdragsmatrisen hentes fra hovedfilen Tofase, 
% og parameterne hentes fra parameterfilen. Denne rutinen er lik 
% for systemet uten tilbakekobling,systemet med trykkregulering 
% og systemet med kaskaderegulering. Denne rutinen er hentet 
% fra (Jahanshahi, 2013) og dens innhold er ikke endret foruten 
% om estetiske endringer. 

% ************ Finner initelle verdier p tilstandene ************

function [x0,y0,par] = MiniLoop_initialize(u,par)

%  x1: Gassen masse i rrledning m_G1
%  x2: Vskens masse i rrledning m_L1
%  x3: Gassen masse i stigerr m_G2
%  x4: Vskens masse i stigerr m_L2

z=u(1);
wG_in=u(2);
wL_in=u(3);
syms P1 x3 x4 %Lager symbolske variable av P1, x3 (m_G2) og x4 (m_L2). 

h1 = 0.003;


P2_t = x3*par.R*par.T2/(par.M_G*(par.V2 - x4/par.rho_L));

rho_G2 = P2_t*par.M_G/(par.R*par.T2);

Fric_pipe = 210.7409;
Fric_riser = 269.0484;

Alpha_L2_av = x4/(par.rho_L*par.V2);

rho_mix_av = (x3+x4)/par.V2;
P2_b = P2_t + rho_mix_av*par.g*par.L2 + Fric_riser;
A_g = par.A1 + par.A1*h1^2/par.hc^2 - 2*h1*par.A1/par.hc;
A_l = par.A1-A_g ;

% Ligning 2 
eq2 = wL_in^2 - par.rho_L*(P1-Fric_pipe+par.rho_L*par.g*h1-P2_b)*(A_l*par.K_o)^2;

Alpha_Lt =  2*Alpha_L2_av - h1/par.hc;
rho_t = Alpha_Lt*par.rho_L + (1-Alpha_Lt)*rho_G2;

% Ligning 3 
eq3 = (wL_in+wG_in)^2 - rho_t*(P2_t-par.P0)*(par.K_pc*z)^2;

% Ligning 4 
eq4 = wL_in/(wG_in + wL_in) - x4/(x4+x3);


% Lser ut ligning 2,3 og 4.  
ss = solve(eq2,eq3,eq4);


if(strcmp('7.4.0.287 (R2007a)',version))
    ind = 3;
else
    ind = 2;
end
P1_ss = real(double(ss.P1(ind)));
x3_ss = real(double(ss.x3(ind)));
x4_ss = real(double(ss.x4(ind)));

rho_G1_n = P1_ss*par.M_G/(par.R*par.T1);
a_new = wL_in*rho_G1_n/(wL_in*rho_G1_n + wG_in*par.rho_L);
a_old = a_new - 0.001; % Sikrer at rutinen kjrer minst n gang.

h1_old = h1;
h1_new = par.k_h*a_new*par.hc;
Pnormal = par.Pnormal;
opt = optimset('Display','off','TolFun',1e-20,'TolX',1e-20,'MaxIter',1e4,'MaxFunEvals',1e10);
y0(1) = 1e-3*(P1_ss - Pnormal);

while(abs(a_new-a_old)>1e-8)
    P1_ss = (1e3*y0(1)+Pnormal);
    rho_G1 = P1_ss*par.M_G/(par.R*par.T1);
    par.Alpha_L1_av = wL_in*rho_G1/(wL_in*rho_G1 + wG_in*par.rho_L);
    x2_ss = par.rho_L*par.V1*a_new + (h1_old - h1_new)*par.A1*par.rho_L*(1-a_new)/sin(par.theta);
    x1_ss = (par.V1 + par.Vb - x2_ss/par.rho_L)*rho_G1;
    
    %x0 lagrer steady-state-verdiene til x1,x2,x3 og x4. 
    
    x0 = [x1_ss;x2_ss;x3_ss;x4_ss]; 
    [x0,fval,exitflag,output,jacobian] = fsolve(@MiniLoop_model0,x0,opt,u,par);
    [~,y0] = MiniLoop_model0(x0,u,par);
    x3_ss = x0(3);
    x4_ss = x0(4);
    rho_G1 = (1e3*y0(1)+Pnormal)*par.M_G/(par.R*par.T1);
    a_old = a_new;
    a_new = wL_in*rho_G1/(wL_in*rho_G1 + wG_in*par.rho_L);
    h1_old = h1_new;
    h1_new = par.k_h*a_new*par.hc;
end

% Skriver ut initelt trykk i rrledning, og i stigerr, samt 
% volumstrmningen gjennom utlpsventilen og massestrmningen 
% gjennom utlpsventilen: 

str1 = ['Initielt trykk i rrledning: ' num2str(y0(1)) ' kpa mlt' ];
disp(str1)
str2 = ['Initielt trykk ved toppen av stigerr:: ' num2str(y0(2)) ' kpa mlt' ];
disp(str2)
str3 = ['Volumstrmning gjennom utlpsventil:: ' num2str(y0(3)) ' litr/min' ];
disp(str3)
str4 = ['Massestrmning gjennom utlpsventil: ' num2str(y0(4)) ' kg/min' ];
disp(str4)

end