
% Dette er hovedfila Tofase for systemet med kaskaderegulering.
% Denne rutinen er utviklet i prosjektet, og baserer seg p
% (Jahanshahi, 2013). Her vises hovedvinduet for det kaskaderegulerte 
% systemet. Her hentes tre andre rutiner: 'Miniloop_parameters()', 
% 'MiniLoop_initialize(u0,par)' og 'Loop_model'.


clear all;
close all;
clc;

% Definerer globale variable

global z1_plot QQ_out_plot Alpha_Lmt_plot Alpha_Lt_plot 
global A_g_plot A_l_plot  

% Henter verdier fra filen MiniLoop_parameters() 
par =  MiniLoop_parameters();

% Her settes simuleringstid Tf, initell ventilpning z0, samt 
% at innstrmningsratene QG_in og QL_in settes konstante. 
% Innstrmningsratene regnes om fra [l/min] til [kg/s]. 
% Deretter settes verdiene inn i en 3x1 pdragsmatrise u0 . 

Tf=1000; %Sluttid
tu_size = Tf+1;
tu = linspace(0,Tf,tu_size); 

z0=10;

QG_in = 4.5; %l/min
QL_in = 4;  %l/min
wG_in0 = par.Rog_n*QG_in/(60*1000); %kg/s
wL_in0 = par.rho_L*QL_in/(60*1000); %kg/s
u0 = [z0;wG_in0;wL_in0];




%********************* Finner initalverdier **********************

%Her kjres initaliseringsfilen MiniLoop_Initialize. Initialiseringsfilen
% bruker parametere fra parameterfilen MiniLoop_parameters og fra matrisen 
% u0. Nr systemet er ferdig initalisert lages det en pdragsmatrise
% med lik lengde som antall steg systemet skal kjres. slik at u blir
% 3x(size(tu)) matrise. Hvor tu er antall steg systemet kjres. 


disp('Initialiserer...')
tic 
[x0,y0,par] =  MiniLoop_initialize(u0,par); 
toc
disp('Ferdig initialisert...')



%******************Sprang i trykkreferansen **********************

% Her settes trykkreferansen P_ref0 lik inngangstrykket par.Pin. Videre
% settes en trykkregulatorens P-forsterkning K_p0 til rett verdi. 
% Deretter defineres matrisene til P_ref0 og K_p0 like lange som lengden
% av tu. Ved  bruke en for-lkke kan man utfre flere sprang i P_ref
% i lpet av samme simulering. Dette gjres ved  legge inn flere 
% "elseif" setninger. 

u =u0*ones(size(tu)); 
P_ref0 = par.Pin;
K_p0 = -1*0.45*7.61e-3;
P_ref =P_ref0*ones(size(tu)); 
K_p = K_p0*ones(size(tu)); 

for i = 1:tu_size
    
    if i < 0.1*Tf*(Tf/tu_size)
         P_ref(i) = P_ref0;
         
     elseif i < 0.3*Tf*(Tf/tu_size) && i >= 0.1*Tf*(Tf/tu_size)
          P_ref(i) = P_ref0 - 0.02e5; 
          
     else
          P_ref(i) = P_ref0-0.04e5;
     end
end


% Her beregnes tilstandens verdier ved  bruke en ode23s-funksjon. 
% Denne metoden passer et system som er 'stiff' og baserer seg p 
% en modifisert Rosenbrock formel av andre orden. 
% ode23s returnerer tiden T og verdiene p tilstandene X

options = odeset ('InitialStep', 1.0e-1, 'MaxStep', 1.0e-1,'RelTol', 1.0e-6, 'AbsTol', 1.0e-6);
[T,X] = ode23s(@(t,x)Loop_model(t,x,tu,u,par,P_ref,K_p),[0,Tf],[x0;0;0;0.1],options);



% Beregner trykket i rrledningen (P1) og p toppen av stigerret (P2_t)

P1 = X(:,1)*par.R*par.T1./(par.M_G*(par.V1 + par.Vb - X(:,2)/par.rho_L));
V_G2 = par.V2 - X(:,4)/par.rho_L;
P2_t = X(:,3)*par.R*par.T2./(par.M_G*V_G2);
P2_t = max(P2_t,par.P0);



% Til slutt plottes figur 1, 2 og 3. 

figure(1)
grid on
plot(T,P1)

figure(2)
grid on
plot(T,P2_t)

figure(3)
plotyy(T(120:180),P1(120:180),T(120:180),P2_t(120:180))
[haxes,hline1,hline2] = plotyy(T,P1,T,P2_t,'plot','plot');
ylabel(haxes(1),'P1 [Pa]')   %Definerer venstre y-akse
ylabel(haxes(2),'P2_t [Pa]') %Definerer hyre y-akse
xlabel(haxes(2),'Tid [sek]') %Definerer x-aksen
legend('y = P1', 'y = P2_t')
